
from dataclasses import dataclass
from typing import Dict
from ..utils.get_mode import is_generate_code_mode
from ..utils.get_s3_client import get_s3_client
@dataclass
class UploadedFile:
    file_name: str
    bucket: str
    region: str
    endpoint: str | None
    suggested_file_name: str

    def __json__(self):
        return self.to_dict()

    def to_dict(self) -> Dict[str, str]:
        return {"file_name": self.file_name, "bucket": self.bucket, "region": self.region, "suggested_file_name": self.suggested_file_name}

    @classmethod
    def from_dict(cls, data: Dict[str, str]) -> "UploadedFile":
        return cls(file_name=data["file_name"], bucket=data["bucket"], region=data["region"], suggested_file_name=data["suggested_file_name"], endpoint=data["endpoint"] or None)

    def get_signed_url(self, expiration: int = 3600 * 24 * 5):
        if is_generate_code_mode():
            return "https://not.real.com"
        s3_client = get_s3_client()

        response = s3_client.generate_presigned_url(
            ClientMethod="get_object",
            Params={"Bucket": self.bucket, "Key": self.file_name},
            ExpiresIn=expiration,
            HttpMethod="GET",
        )
        return response

    def get_s3_key(self):
        if self.should_upload_to_r2():
            raise Exception("get_s3_key function is not supported when using a custom s3 endpoint")

        return f"https://{self.bucket}.s3.{self.region}.amazonaws.com/{self.file_name}"

    def get_file_path(self):
        return self.file_name

    def should_upload_to_r2(self):
        if isinstance(self.endpoint, str) and self.endpoint == "":
            return False
        return True